@@ -5,7 +5,7 @@ warnings.filterwarnings("ignore", category=FutureWarning)
 
 try:
     from ._version import version as __version__
-except ImportError:
+except ImportError:  # pragma: no cover
     __version__ = "unknown"
 
 
