@@ -3,6 +3,8 @@ from collections import defaultdict
 from datetime import datetime
 from functools import partial
 from math import ceil, log10
+import pandas as pd
+from pathlib import Path
 from types import MethodType
 from typing import Optional, Sequence, Union
 
@@ -13,8 +15,8 @@ from napari.layers.points._points_key_bindings import register_points_action
 from napari.layers.utils import color_manager
 from napari.utils.events import Event
 from napari.utils.history import get_save_history, update_save_history
-from qtpy.QtCore import Qt, QTimer, Signal
-from qtpy.QtGui import QPainter
+from qtpy.QtCore import Qt, QTimer, Signal, QSize
+from qtpy.QtGui import QPainter, QIcon
 from qtpy.QtWidgets import (
     QButtonGroup,
     QCheckBox,
@@ -36,8 +38,12 @@ from qtpy.QtWidgets import (
 
 from napari_deeplabcut import keypoints
 from napari_deeplabcut._reader import _load_config
-from napari_deeplabcut._writer import _write_config, _write_image
-from napari_deeplabcut.misc import encode_categories, to_os_dir_sep
+from napari_deeplabcut._writer import _write_config, _write_image, _form_df
+from napari_deeplabcut.misc import (
+    encode_categories,
+    to_os_dir_sep,
+    guarantee_multiindex_rows,
+)
 
 
 def _get_and_try_preferred_reader(
@@ -114,13 +120,16 @@ def _save_layers_dialog(self, selected=False):
 
 def on_close(self, event, widget):
     if widget._stores and not widget._is_saved:
-        QMessageBox.warning(
+        choice = QMessageBox.warning(
             widget,
-            "",
-            "Please save your data before closing",
-            QMessageBox.Ok,
+            "Warning",
+            "Data were not saved. Are you certain you want to leave?",
+            QMessageBox.Yes | QMessageBox.No,
         )
-        event.ignore()
+        if choice == QMessageBox.Yes:
+            event.accept()
+        else:
+            event.ignore()
     else:
         event.accept()
 
@@ -142,7 +151,7 @@ class KeypointControls(QWidget):
         status_bar = self.viewer.window._qt_window.statusBar()
         self.last_saved_label = QLabel("")
         self.last_saved_label.hide()
-        status_bar.insertPermanentWidget(0, self.last_saved_label)
+        status_bar.addPermanentWidget(self.last_saved_label)
 
         # Hack napari's Welcome overlay to show more relevant instructions
         overlay = self.viewer.window._qt_viewer._canvas_overlay
@@ -194,8 +203,8 @@ class KeypointControls(QWidget):
 
         self._radio_group = self._form_mode_radio_buttons()
 
+        self._display = ColorSchemeDisplay(parent=self)
         self._color_scheme_display = self._form_color_scheme_display(self.viewer)
-
         self._view_scheme_cb.toggled.connect(self._show_color_scheme)
         self._view_scheme_cb.toggle()
 
@@ -254,19 +263,41 @@ class KeypointControls(QWidget):
         return extract_button, crop_button
 
     def _extract_single_frame(self, *args):
-        layer = None
-        for layer_ in self.viewer.layers:
-            if isinstance(layer_, Image):
-                layer = layer_
-                break
-        if layer is not None:
+        image_layer = None
+        points_layer = None
+        for layer in self.viewer.layers:
+            if isinstance(layer, Image):
+                image_layer = layer
+            elif isinstance(layer, Points):
+                points_layer = layer
+        if image_layer is not None:
             ind = self.viewer.dims.current_step[0]
-            frame = layer.data[ind]
-            n_frames = layer.data.shape[0]
+            frame = image_layer.data[ind]
+            n_frames = image_layer.data.shape[0]
             name = f"img{str(ind).zfill(int(ceil(log10(n_frames))))}.png"
-            output_path = os.path.join(layer.metadata["root"], name)
+            output_path = os.path.join(image_layer.metadata["root"], name)
             _write_image(frame, str(output_path))
 
+            # If annotations were loaded, they should be written to a machinefile.h5 file
+            if points_layer is not None:
+                df = _form_df(
+                    points_layer.data,
+                    {
+                        "metadata": points_layer.metadata,
+                        "properties": points_layer.properties,
+                    },
+                )
+                df = df.iloc[ind:ind + 1]
+                df.index = pd.MultiIndex.from_tuples([Path(output_path).parts[-3:]])
+                filepath = os.path.join(image_layer.metadata["root"], "machinelabels-iter0.h5")
+                if Path(filepath).is_file():
+                    df_prev = pd.read_hdf(filepath)
+                    guarantee_multiindex_rows(df_prev)
+                    df = pd.concat([df_prev, df])
+                    df = df[~df.index.duplicated(keep="first")]
+                df.to_hdf(filepath, key="machinelabels")
+
+
     def _store_crop_coordinates(self, *args):
         if not (project_path := self._images_meta.get("project")):
             return
@@ -293,6 +324,11 @@ class KeypointControls(QWidget):
 
     def _form_dropdown_menus(self, store):
         menu = KeypointsDropdownMenu(store)
+        self.viewer.dims.events.current_step.connect(
+            menu.smart_reset,
+            position="last",
+        )
+        menu.smart_reset(event=None)
         self._menus.append(menu)
         layout = QVBoxLayout()
         layout.addWidget(menu)
@@ -318,29 +354,23 @@ class KeypointControls(QWidget):
         return group
 
     def _form_color_scheme_display(self, viewer):
-        display = ColorSchemeDisplay(parent=self)
-        self._update_color_scheme(display)
-
-        self.viewer.layers.events.inserted.connect(
-            partial(self._update_color_scheme, display)
-        )
-
+        self.viewer.layers.events.inserted.connect(self._update_color_scheme)
         return viewer.window.add_dock_widget(
-            display, name="Color scheme reference", area="left"
+            self._display, name="Color scheme reference", area="left"
         )
 
-    def _update_color_scheme(self, display):
+    def _update_color_scheme(self):
+        def to_hex(nparray):
+            a = np.array(nparray * 255, dtype=int)
+            rgb2hex = lambda r, g, b, _: f"#{r:02x}{g:02x}{b:02x}"
+            res = rgb2hex(*a)
+            return res
+
+        self._display.reset()
         for layer in self.viewer.layers:
             if isinstance(layer, Points) and layer.metadata:
-
-                def to_hex(nparray):
-                    a = np.array(nparray * 255, dtype=int)
-                    rgb2hex = lambda r, g, b, _: f"#{r:02x}{g:02x}{b:02x}"
-                    res = rgb2hex(*a)
-                    return res
-
                 [
-                    display.add_entry(name, to_hex(color))
+                    self._display.add_entry(name, to_hex(color))
                     for name, color in layer.metadata["face_color_cycles"][
                         "label"
                     ].items()
@@ -403,6 +433,38 @@ class KeypointControls(QWidget):
                 10, partial(self._move_image_layer_to_bottom, event.index)
             )
         elif isinstance(layer, Points):
+            # If the current Points layer comes from a config file, some have already
+            # been added and the body part names are different from the existing ones,
+            # then we update store's metadata and menus.
+            if layer.metadata.get("project", "") and self._stores:
+                keypoints_menu = self._menus[0].menus["label"]
+                current_keypoint_set = set(
+                    keypoints_menu.itemText(i) for i in range(keypoints_menu.count())
+                )
+                new_keypoint_set = set(layer.metadata["header"].bodyparts)
+                diff = new_keypoint_set.difference(current_keypoint_set)
+                if diff:
+                    answer = QMessageBox.question(self, "", "Do you want to display the new keypoints only?")
+                    if answer  == QMessageBox.Yes:
+                        self.viewer.layers[-2].shown = False
+
+                    self.viewer.status = f"New keypoint{'s' if len(diff) > 1 else ''} {', '.join(diff)} found."
+                    for _layer, store in self._stores.items():
+                        _layer.metadata["header"] = layer.metadata["header"]
+                        _layer.metadata["face_color_cycles"] = layer.metadata["face_color_cycles"]
+                        _layer.face_color_cycle = layer.face_color_cycle
+                        store.layer = _layer
+
+                    for menu in self._menus:
+                        menu._map_individuals_to_bodyparts()
+                        menu._update_items()
+
+                    self._update_color_scheme()
+
+                # Remove the unnecessary layer newly added
+                QTimer.singleShot(10, self.viewer.layers.pop)
+                return
+
             store = keypoints.KeypointStore(self.viewer, layer)
             self._stores[layer] = store
             # TODO Set default dir of the save file dialog
@@ -416,11 +478,7 @@ class KeypointControls(QWidget):
             layer.events.query_next_frame.connect(store._advance_step)
             layer.bind_key("Shift-Right", store._find_first_unlabeled_frame)
             layer.bind_key("Shift-Left", store._find_first_unlabeled_frame)
-            self.viewer.dims.events.current_step.connect(
-                store.smart_reset,
-                position="last",
-            )
-            store.smart_reset(event=None)
+
             layer.bind_key("Down", store.next_keypoint, overwrite=True)
             layer.bind_key("Up", store.prev_keypoint, overwrite=True)
             layer.face_color_mode = "cycle"
@@ -438,14 +496,15 @@ class KeypointControls(QWidget):
 
     def on_remove(self, event):
         layer = event.value
-        if isinstance(layer, Points):
+        n_points_layer = sum(isinstance(l, Points) for l in self.viewer.layers)
+        if isinstance(layer, Points) and n_points_layer == 0:
             if self._color_scheme_display is not None:
-                self.viewer.window.remove_dock_widget(self._color_scheme_display)
+                self._display.reset()
             self._stores.pop(layer, None)
             while self._menus:
                 menu = self._menus.pop()
                 self._layout.removeWidget(menu)
-                menu.setParent(None)
+                menu.deleteLater()
                 menu.destroy()
             self._trail_cb.setEnabled(False)
             self.last_saved_label.hide()
@@ -497,7 +556,7 @@ def toggle_edge_color(layer):
 class DropdownMenu(QComboBox):
     def __init__(self, labels: Sequence[str], parent: Optional[QWidget] = None):
         super().__init__(parent)
-        self.addItems(labels)
+        self.update_items(labels)
 
     def update_to(self, text: str):
         index = self.findText(text)
@@ -507,6 +566,10 @@ class DropdownMenu(QComboBox):
     def reset(self):
         self.setCurrentIndex(0)
 
+    def update_items(self, items):
+        self.clear()
+        self.addItems(items)
+
 
 class KeypointsDropdownMenu(QWidget):
     def __init__(
@@ -517,33 +580,60 @@ class KeypointsDropdownMenu(QWidget):
         super().__init__(parent)
         self.store = store
         self.store.layer.events.current_properties.connect(self.update_menus)
+        self._locked = False
 
-        # Map individuals to their respective bodyparts
         self.id2label = defaultdict(list)
-        for keypoint in store._keypoints:
-            label = keypoint.label
-            id_ = keypoint.id
-            if label not in self.id2label[id_]:
-                self.id2label[id_].append(label)
-
         self.menus = dict()
-        if store.ids[0]:
-            menu = create_dropdown_menu(store, list(self.id2label), "id")
-            menu.currentTextChanged.connect(self.refresh_label_menu)
-            self.menus["id"] = menu
-        self.menus["label"] = create_dropdown_menu(
-            store, self.id2label[store.ids[0]], "label"
-        )
+        self._map_individuals_to_bodyparts()
+        self._populate_menus()
+
         layout1 = QVBoxLayout()
         layout1.addStretch(1)
         group_box = QGroupBox("Keypoint selection")
         layout2 = QVBoxLayout()
         for menu in self.menus.values():
             layout2.addWidget(menu)
+        self.lock_button = QPushButton("Lock selection")
+        self.lock_button.setIcon(QIcon('src/napari_deeplabcut/assets/unlock.svg'))
+        self.lock_button.setIconSize(QSize(24, 24))
+        self.lock_button.clicked.connect(self._lock_current_keypoint)
+        layout2.addWidget(self.lock_button)
         group_box.setLayout(layout2)
         layout1.addWidget(group_box)
         self.setLayout(layout1)
 
+    def _map_individuals_to_bodyparts(self):
+        for keypoint in self.store._keypoints:
+            label = keypoint.label
+            id_ = keypoint.id
+            if label not in self.id2label[id_]:
+                self.id2label[id_].append(label)
+
+    def _populate_menus(self):
+        id_ = self.store.ids[0]
+        if id_:
+            menu = create_dropdown_menu(self.store, list(self.id2label), "id")
+            menu.currentTextChanged.connect(self.refresh_label_menu)
+            self.menus["id"] = menu
+        self.menus["label"] = create_dropdown_menu(
+            self.store, self.id2label[id_], "label",
+        )
+
+    def _update_items(self):
+        id_ = self.store.ids[0]
+        if id_:
+            self.menus["id"].update_items(list(self.id2label))
+        self.menus["label"].update_items(self.id2label[id_])
+
+    def _lock_current_keypoint(self):
+        self._locked = not self._locked
+        if self._locked:
+            self.lock_button.setText("Unlock selection")
+            self.lock_button.setIcon(QIcon('src/napari_deeplabcut/assets/lock.svg'))
+        else:
+            self.lock_button.setText("Lock selection")
+            self.lock_button.setIcon(QIcon('src/napari_deeplabcut/assets/unlock.svg'))
+
     def update_menus(self, event):
         keypoint = self.store.current_keypoint
         for attr, menu in self.menus.items():
@@ -558,6 +648,18 @@ class KeypointsDropdownMenu(QWidget):
         menu.blockSignals(False)
         menu.addItems(self.id2label[text])
 
+    def smart_reset(self, event):
+        """Set current keypoint to the first unlabeled one."""
+        if self._locked:
+            return
+        unannotated = ""
+        already_annotated = self.store.annotated_keypoints
+        for keypoint in self.store._keypoints:
+            if keypoint not in already_annotated:
+                unannotated = keypoint
+                break
+        self.store.current_keypoint = unannotated if unannotated else self.store._keypoints[0]
+
 
 def create_dropdown_menu(store, items, attr):
     menu = DropdownMenu(items)
@@ -774,5 +876,8 @@ class ColorSchemeDisplay(QScrollArea):
         self._layout.addWidget(
             LabelPair(color, name, self), alignment=Qt.AlignmentFlag.AlignLeft
         )
-        self._container.setLayout(self._layout)
-        self._container.update()
+
+    def reset(self):
+        self.scheme_dict = {}
+        for i in reversed(range(self._layout.count())):
+            self._layout.itemAt(i).widget().deleteLater()
