@@ -31,6 +31,9 @@ class LabelMode(CycleEnum):
         annotated point actually moves it to the cursor location.
     LOOP: the first point is placed frame by frame, then it wraps
         to the next label at the end and restart from frame 1, etc.
+        Unless the keypoint selection is locked, the dropdown menu is
+        automatically set to the first unlabeled keypoint of
+        the current frame.
     """
 
     SEQUENTIAL = auto()
@@ -49,7 +52,10 @@ TOOLTIPS = {
     "QUICK": "Similar to SEQUENTIAL, but trying to add an already\n"
     "annotated point actually moves it to the cursor location.",
     "LOOP": "The first point is placed frame by frame, then it wraps\n"
-    "to the next label at the end and restart from frame 1, etc.",
+    "to the next label at the end and restart from frame 1, etc.\n"
+    "Unless the keypoint selection is locked, the dropdown menu is\n"
+    "automatically set to the first unlabeled keypoint of\n"
+    "the current frame.",
 }
 
 
@@ -59,12 +65,21 @@ Keypoint = namedtuple("Keypoint", ["label", "id"])
 class KeypointStore:
     def __init__(self, viewer, layer: Points):
         self.viewer = viewer
+        self._keypoints = []
         self.layer = layer
+        self.viewer.dims.set_current_step(0, 0)
+
+    @property
+    def layer(self):
+        return self._layer
+
+    @layer.setter
+    def layer(self, layer):
+        self._layer = layer
         all_pairs = self.layer.metadata["header"].form_individual_bodypart_pairs()
         self._keypoints = [
             Keypoint(label, id_) for id_, label in all_pairs
         ]  # Ordered references to all possible keypoints
-        self.viewer.dims.set_current_step(0, 0)
 
     @property
     def current_step(self):
@@ -99,16 +114,6 @@ class KeypointStore:
             current_properties["id"] = np.asarray([keypoint.id])
             self.layer.current_properties = current_properties
 
-    def smart_reset(self, event):
-        """Set current keypoint to the first unlabeled one."""
-        unannotated = ""
-        already_annotated = self.annotated_keypoints
-        for keypoint in self._keypoints:
-            if keypoint not in already_annotated:
-                unannotated = keypoint
-                break
-        self.current_keypoint = unannotated if unannotated else self._keypoints[0]
-
     def next_keypoint(self, *args):
         ind = self._keypoints.index(self.current_keypoint) + 1
         if ind <= len(self._keypoints) - 1:
